@@ -12,8 +12,6 @@ from astropy import wcs as astwcs
 from astropy.wcs import wcsapi
 from astropy.time import Time
 
-from gwcs.wcs import new_bbox
-
 from .. import wcs
 from ..wcstools import (wcs_from_fiducial, grid_from_bounding_box, wcs_from_points)
 from .. import coordinate_frames as cf
@@ -32,6 +30,7 @@ detector = cf.Frame2D(name='detector', axes_order=(0, 1))
 focal = cf.Frame2D(name='focal', axes_order=(0, 1), unit=(u.m, u.m))
 spec = cf.SpectralFrame(name='wave', unit=[u.m, ], axes_order=(2, ), axes_names=('lambda', ))
 time = cf.TemporalFrame(name='time', unit=[u.s, ], axes_order=(3, ), axes_names=('time', ), reference_frame=Time("2020-01-01"))
+stokes = cf.StokesFrame(axes_order=(2,))
 
 pipe = [wcs.Step(detector, m1),
         wcs.Step(focal, m2),
@@ -81,7 +80,7 @@ def test_init_no_transform():
     assert gw.pipeline[1].frame == "icrs"
     with pytest.warns(DeprecationWarning, match="Indexing a WCS.pipeline step is deprecated."):
         assert gw.pipeline[1][0] == "icrs"
-    assert np.in1d(gw.available_frames, ['detector', 'icrs']).all()
+    assert np.isin(gw.available_frames, ['detector', 'icrs']).all()
     gw = wcs.WCS(output_frame=icrs, input_frame=detector)
     assert gw._pipeline[0].frame == "detector"
     with pytest.warns(DeprecationWarning, match="Indexing a WCS.pipeline step is deprecated."):
@@ -89,7 +88,7 @@ def test_init_no_transform():
     assert gw._pipeline[1].frame == "icrs"
     with pytest.warns(DeprecationWarning, match="Indexing a WCS.pipeline step is deprecated."):
         assert gw._pipeline[1][0] == "icrs"
-    assert np.in1d(gw.available_frames, ['detector', 'icrs']).all()
+    assert np.isin(gw.available_frames, ['detector', 'icrs']).all()
     with pytest.raises(NotImplementedError):
         gw(1, 2)
 
@@ -181,6 +180,16 @@ def test_backward_transform():
     assert_allclose(w.backward_transform(1, 2), (-3, 1))
 
 
+def test_backward_transform_has_inverse():
+    """
+    Test that backward transform has an inverse, which is the forward transform
+    """
+    poly = models.Polynomial1D(1, c0=4)
+    poly.inverse = models.Polynomial1D(1, c0=-3)  # this is NOT the actual inverse of poly
+    w = wcs.WCS(forward_transform=poly & models.Scale(2), output_frame='sky')
+    assert_allclose(w.backward_transform.inverse(1, 2), w(1, 2))
+
+
 def test_return_coordinates():
     """Test converting to coordinate objects or quantities."""
     w = wcs.WCS(pipe[:])
@@ -213,6 +222,15 @@ def test_return_coordinates():
     output_quant = w.output_frame.coordinate_to_quantity(*num_plus_output)
     assert_allclose(utils.get_values(w.unit, *output_quant), numerical_result)
 
+    # CompositeFrame - [celestial, Stokes]
+    output_frame = cf.CompositeFrame(frames=[icrs, stokes])
+    transform = m1 & models.Identity(1)
+    w = wcs.WCS(forward_transform=transform, output_frame=output_frame)
+    numerical_result = transform(x, y, y)
+    num_plus_output = w(x, y, y, with_units=True)
+    output_quant = w.output_frame.coordinate_to_quantity(*num_plus_output)
+    assert_allclose(utils.get_values(w.unit, *output_quant), numerical_result)
+
 
 def test_from_fiducial_sky():
     sky = coord.SkyCoord(1.63 * u.radian, -72.4 * u.deg, frame='fk5')
@@ -267,10 +285,7 @@ def test_bounding_box():
     pipeline = [('detector', trans2), ('sky', None)]
     w = wcs.WCS(pipeline)
     w.bounding_box = bb
-    if new_bbox:
-        assert w.bounding_box == w.forward_transform.bounding_box
-    else:
-        assert w.bounding_box == w.forward_transform.bounding_box[::-1]
+    assert w.bounding_box == w.forward_transform.bounding_box
 
     pipeline = [("detector", models.Shift(2)), ("sky", None)]
     w = wcs.WCS(pipeline)
@@ -297,29 +312,24 @@ def test_compound_bounding_box():
         2: ((-1, 5), (3, 17)),
         3: ((-3, 7), (1, 27)),
     }
-    if new_bbox:
-        # Test attaching a valid bounding box (ignoring input 'x')
-        w.attach_compound_bounding_box(cbb, [('x',)])
-        from astropy.modeling.bounding_box import CompoundBoundingBox
-        cbb = CompoundBoundingBox.validate(trans3, cbb, selector_args=[('x',)], order='F')
-        assert w.bounding_box == cbb
-        assert w.bounding_box is trans3.bounding_box
+    # Test attaching a valid bounding box (ignoring input 'x')
+    w.attach_compound_bounding_box(cbb, [('x',)])
+    from astropy.modeling.bounding_box import CompoundBoundingBox
+    cbb = CompoundBoundingBox.validate(trans3, cbb, selector_args=[('x',)], order='F')
+    assert w.bounding_box == cbb
+    assert w.bounding_box is trans3.bounding_box
 
-        # Test evaluating
-        assert_allclose(w(13, 2, 1), (np.nan, np.nan, np.nan))
-        assert_allclose(w(13, 2, 2), (np.nan, np.nan, np.nan))
-        assert_allclose(w(13, 0, 3), (np.nan, np.nan, np.nan))
-        # No bounding box for selector
-        with pytest.raises(RuntimeError):
-            w(13, 13, 4)
+    # Test evaluating
+    assert_allclose(w(13, 2, 1), (np.nan, np.nan, np.nan))
+    assert_allclose(w(13, 2, 2), (np.nan, np.nan, np.nan))
+    assert_allclose(w(13, 0, 3), (np.nan, np.nan, np.nan))
+    # No bounding box for selector
+    with pytest.raises(RuntimeError):
+        w(13, 13, 4)
 
-        # Test attaching a invalid bounding box (not ignoring input 'x')
-        with pytest.raises(ValueError):
-            w.attach_compound_bounding_box(cbb, [('x', False)])
-    else:
-        with pytest.raises(NotImplementedError) as err:
-            w.attach_compound_bounding_box(cbb, [('x',)])
-        assert str(err.value) == 'Compound bounding box is not supported for your version of astropy'
+    # Test attaching a invalid bounding box (not ignoring input 'x')
+    with pytest.raises(ValueError):
+        w.attach_compound_bounding_box(cbb, [('x', False)])
 
     # Test that bounding_box with quantities can be assigned and evaluates
     trans = models.Shift(10 * u .pix) & models.Shift(2 * u.pix)
@@ -329,19 +339,15 @@ def test_compound_bounding_box():
         1 * u.pix: (1 * u.pix, 5 * u.pix),
         2 * u.pix: (2 * u.pix, 6 * u.pix)
     }
-    if new_bbox:
-        w.attach_compound_bounding_box(cbb, [('x1',)])
+    w.attach_compound_bounding_box(cbb, [('x1',)])
 
-        from astropy.modeling.bounding_box import CompoundBoundingBox
-        cbb = CompoundBoundingBox.validate(trans, cbb, selector_args=[('x1',)], order='F')
-        assert w.bounding_box == cbb
-        assert w.bounding_box is trans.bounding_box
+    from astropy.modeling.bounding_box import CompoundBoundingBox
+    cbb = CompoundBoundingBox.validate(trans, cbb, selector_args=[('x1',)], order='F')
+    assert w.bounding_box == cbb
+    assert w.bounding_box is trans.bounding_box
 
-        assert_allclose(w(-1*u.pix, 1*u.pix), (np.nan, np.nan))
-        assert_allclose(w(7*u.pix, 2*u.pix), (np.nan, np.nan))
-    else:
-        with pytest.raises(NotImplementedError) as err:
-            w.attach_compound_bounding_box(cbb, [('x1',)])
+    assert_allclose(w(-1*u.pix, 1*u.pix), (np.nan, np.nan))
+    assert_allclose(w(7*u.pix, 2*u.pix), (np.nan, np.nan))
 
 
 def test_grid_from_bounding_box():
@@ -634,8 +640,9 @@ def test_to_fits_sip():
     y, x = np.mgrid[:1024:10, :1024:10]
     xflat = np.ravel(x[1:-1, 1:-1])
     yflat = np.ravel(y[1:-1, 1:-1])
-    af = asdf.open(get_pkg_data_filename('data/miriwcs.asdf'))
-    miriwcs = af.tree['wcs']
+    fn = get_pkg_data_filename('data/miriwcs.asdf')
+    with asdf.open(fn, lazy_load=False, copy_arrays=True, ignore_missing_extensions=True) as af:
+        miriwcs = af.tree['wcs']
     bounding_box = ((0, 1024), (0, 1024))
     mirisip = miriwcs.to_fits_sip(bounding_box, max_inv_pix_error=0.1, verbose=True)
     fitssip = astwcs.WCS(mirisip)
@@ -920,10 +927,7 @@ def test_to_fits_1D_round_trip(gwcs_1d_spectral):
 
     # test points:
     np.random.seed(1)
-    if new_bbox:
-        (xmin, xmax) = w.bounding_box.bounding_box()
-    else:
-        (xmin, xmax) = w.bounding_box
+    (xmin, xmax) = w.bounding_box.bounding_box()
     x = xmin + (xmax - xmin) * np.random.random(100)
 
     # test forward transformation:
@@ -997,8 +1001,9 @@ def test_to_fits_tab_time_cube(gwcs_cube_with_separable_time):
 
 def test_to_fits_tab_miri_image():
     # gWCS:
-    af = asdf.open(get_pkg_data_filename('data/miriwcs.asdf'))
-    w = af.tree['wcs']
+    fn = get_pkg_data_filename('data/miriwcs.asdf')
+    with asdf.open(fn, copy_arrays=True, lazy_load=False, ignore_missing_extensions=True) as af:
+        w = af.tree['wcs']
 
     # FITS WCS -TAB:
     hdr, bt = w.to_fits_tab(sampling=0.5)
@@ -1020,8 +1025,9 @@ def test_to_fits_tab_miri_image():
 
 
 def test_to_fits_tab_miri_lrs():
-    af = asdf.open(get_pkg_data_filename('data/miri_lrs_wcs.asdf'))
-    w = af.tree['wcs']
+    fn = get_pkg_data_filename('data/miri_lrs_wcs.asdf')
+    with asdf.open(fn, copy_arrays=True, lazy_load=False, ignore_missing_extensions=True) as af:
+        w = af.tree['wcs']
 
     # FITS WCS -TAB:
     hdr, bt = w.to_fits(sampling=0.25)
@@ -1088,7 +1094,9 @@ def test_in_image():
 
 
 def test_iter_inv():
-    w = asdf.open(get_pkg_data_filename('data/nircamwcs.asdf')).tree['wcs']
+    fn = get_pkg_data_filename('data/nircamwcs.asdf')
+    with asdf.open(fn, lazy_load=False, copy_arrays=False, ignore_missing_extensions=True) as af:
+        w = af.tree['wcs']
     # remove analytic/user-supplied inverse:
     w.pipeline[0].transform.inverse = None
     w.bounding_box = None
@@ -1114,7 +1122,8 @@ def test_iter_inv():
     )
     assert np.allclose((x, y), (xp, yp))
 
-    w = asdf.open(get_pkg_data_filename('data/nircamwcs.asdf')).tree['wcs']
+    with asdf.open(fn, lazy_load=False, copy_arrays=False, ignore_missing_extensions=True) as af:
+        w = af.tree['wcs']
 
     # test single point
     assert np.allclose((1, 2), w.numerical_inverse(*w(1, 2)))
@@ -1188,7 +1197,7 @@ def test_iter_inv():
 
 def test_tabular_2d_quantity():
     shape = (3, 3)
-    data = np.arange(np.product(shape)).reshape(shape) * u.m / u.s
+    data = np.arange(np.prod(shape)).reshape(shape) * u.m / u.s
 
     # The integer location is at the centre of the pixel.
     points_unit = u.pix
@@ -1234,7 +1243,9 @@ def test_sip_roundtrip():
     hdr['naxis'] = 2
     hdr['naxis1'] = nx
     hdr['naxis2'] = ny
-    gw = _gwcs_from_hst_fits_wcs(hdr)
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", category=astwcs.FITSFixedWarning)
+        gw = _gwcs_from_hst_fits_wcs(hdr)
     hdr_back = gw.to_fits_sip(
         max_pix_error=1e-6,
         max_inv_pix_error=None,
@@ -1260,3 +1271,41 @@ def test_sip_roundtrip():
                         atol=0.0,
                         rtol=1.0e-8 * 10**(i + j)
                     )
+
+
+def test_spatial_spectral_stokes():
+    """ Converts a FITS WCS to GWCS and compares results."""
+    hdr = fits.Header.fromfile(get_pkg_data_filename("data/stokes.txt"))
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", category=astwcs.FITSFixedWarning)
+        aw = astwcs.WCS(hdr)
+    crpix = aw.wcs.crpix
+    crval = aw.wcs.crval
+    cdelt = aw.wcs.cdelt
+
+    fk5 = cf.CelestialFrame(reference_frame=coord.FK5(), name='FK5')
+    detector = cf.Frame2D(name='detector', axes_order=(0, 1))
+    spec = cf.SpectralFrame(name='FREQ', unit=[u.Hz, ], axes_order=(2, ), axes_names=('freq', ))
+    stokes = cf.StokesFrame(axes_order=(3,))
+    world = cf.CompositeFrame(frames=[fk5, spec, stokes])
+
+    det2sky = (models.Shift(-crpix[0]) & models.Shift(-crpix[1]) |
+               models.Scale(cdelt[0]) & models.Scale(cdelt[1]) |
+               models.Pix2Sky_SIN() | models.RotateNative2Celestial(crval[0], crval[1], 180))
+    det2freq = models.Shift(-crpix[2]) | models.Scale(cdelt[2]) | models.Shift(crval[2])
+    det2stokes = models.Shift(-crpix[3]) | models.Scale(cdelt[3]) | models.Shift(crval[3])
+
+    gw = wcs.WCS([wcs.Step(detector, det2sky & det2freq & det2stokes),
+                  wcs.Step(world, None)]
+                )
+
+    x1 = np.array([0, 0, 0, 0, 0])
+    x2 = np.array([0, 1, 2, 3, 4])
+
+    gw_sky, gw_spec, gw_stokes = gw.pixel_to_world(x1+1, x1+1, x1+1, x2+1)
+    aw_sky, aw_spec, aw_stokes = aw.pixel_to_world(x1, x1, x1, x2)
+
+    assert_allclose(gw_sky.data.lon, aw_sky.data.lon)
+    assert_allclose(gw_sky.data.lat, aw_sky.data.lat)
+    assert_allclose(gw_spec.value, aw_spec.value)
+    assert_allclose(gw_stokes.value, aw_stokes.value)
